@@ -442,11 +442,12 @@ def _wham(**args):
          formatter_class=argparse.ArgumentDefaultsHelpFormatter)
 @arg("-d", "--dry-run", action='store_true',
      help="show command instead of executing it")
+@arg("--lab", help="use jupyterlab", action="store_true")
 @arg("--port", help="set port", default=0)
 @arg("--screen-cmd", help="screen command", default="screen")
 @arg("--screen-log", help="screen logfile (no logfile by default)", default="")
 @arg("--python-exec", help="python executable", default="")
-@arg("-S", "--sockname", help="screen sockname", default="(path):jupyter-server:(port)")
+@arg("-S", "--sockname", help="screen sockname", default="(path):(port):jupyter")
 @arg("--no-screen", help="do not run screen", action="store_true")
 @arg("--keep-ld-library-path", help="(ignored, this is the default now)", action="store_true")
 @arg("--detach", help="detach screen", action="store_true")
@@ -462,9 +463,19 @@ def _jrun(**kargs):
        option) and a jupyter server is then run  inside a `screen` instance.
        You will thus have to type `CTRL+aCTRL+d` in order to detach the screen
        letting it run in the background.
-       To avoid this you can use the `--no-screen` flag. In this case
-       however make sure you do not close the terminal where the server is
-       running.
+
+       Alternatively, you can immediately detach the screen with
+       ```bash
+       cd /path/to/your/notebook/dir
+       bussilab jrun --detach
+       ```
+
+       Notice that, since the server is run inside a screen instance, in order
+       to visualize python outputs that has been sent directly to the terminal you
+       should connect to the screen instance later. By default, a socket name
+       containing the path where the server is running is used, with `/` replaced
+       by `:`. It should thus be easy to use `screen -ls` to find the proper screen
+       instance.
     """
     from . import jremote
     if kargs["keep_ld_library_path"]:
@@ -570,6 +581,8 @@ def _notify(**kargs):
 @command("cron", help="Run cron",
           formatter_class=argparse.ArgumentDefaultsHelpFormatter)
 @arg("--quick-start", help="run immediately", action="store_true")
+@arg("--quick-start-skip-steps", help="skip steps on quick start", default=None, type=int)
+@arg("--quick-start-event", help="event number for quick start", default=None, type=int)
 @arg("--cron-file", help="path to cron file")
 @arg("--screen-cmd", help="screen command", default="screen")
 @arg("--screen-log", help="screen logfile (no logfile by default)", default="")
@@ -580,7 +593,7 @@ def _notify(**kargs):
 @arg("--detach", help="detach screen", action="store_true")
 @arg("--unique", help="allow only one screen with this socket", action="store_true")
 @arg("--window", help="run a new window within the same screen", action="store_true")
-@arg("--period", help="period (seconds)", default=3600, type=int)
+@arg("--period", help="period (seconds) default read from cron file or set to 3600", default=None, type=int)
 @arg("--max-times", help="maximum number of calls", default=None, type=int)
 def _cron(**kargs):
     from . import cron
