@@ -7,15 +7,10 @@ from typing import Optional
 import networkx
 import numpy as np
 
-try:
-    from numba import jit as numba_jit
-except ImportError:
-    import warnings
-    warnings.warn("There was a problem importing numba, jit functions will work but will be MUCH slower")
-    def numba_jit(x):
-        return x
-
 from .coretools import Result
+from .coretools import import_numba_jit
+
+numba_jit=import_numba_jit()
 
 class ClusteringResult(Result):
     """Result of a `bussilab.clustering` calculation."""
@@ -186,12 +181,27 @@ def daura(adj,weights=None,*,min_size=0,max_clusters=None):
         indexes=np.delete(indexes,ii)
     return ClusteringResult(method="daura",clusters=clusters, weights=ww)
 
+@numba_jit
+def _qt_outer(distances,next_,cutoff):
+    candidates=np.empty(len(distances),dtype='int')
+    n=0
+    for i in range(next_):
+        if distances[next_,i]<cutoff:
+           candidates[n]=i
+           n+=1
+    for i in range(next_+1,len(distances)):
+        if distances[next_,i]<cutoff:
+           candidates[n]=i
+           n+=1
+    return candidates[:n]
+
 @numba_jit
 def _qt_inner(distances,dist_from_cluster,candidates,cutoff,weights):
+    if len(dist_from_cluster)<1:
+        return (-1,np.inf)
     next_i=0
     minval=dist_from_cluster[0]
     weight_minval=weights[candidates[0]]
-    weight_minval=0.0
     N=len(dist_from_cluster)
     for i in range(1,N):
         val=dist_from_cluster[i]
@@ -301,9 +311,8 @@ def qt(distances,cutoff,weights=None,*,min_size=0,max_clusters=None):
             next_=sorted_indexes[i_degrees]
             precluster = [next_]
             new_cluster_diameter=0.0
-            candidates=np.where(distances[next_]<cutoff)[0]
+            candidates=_qt_outer(distances,next_,cutoff)
             dist_from_cluster=distances[next_][candidates]
-            dist_from_cluster[np.searchsorted(candidates,next_)]=np.inf
             while True:
                 (next_,minval)=_qt_inner(distances,dist_from_cluster,candidates,cutoff,weights)
                 if(next_<0):
@@ -331,8 +340,9 @@ def qt(distances,cutoff,weights=None,*,min_size=0,max_clusters=None):
             if ncluster >= max_clusters:
                 break
 
-        distances=np.delete(distances,cluster,axis=0)
-        distances=np.delete(distances,cluster,axis=1)
-        weights=np.delete(weights,cluster)
-        indexes=np.delete(indexes,cluster)
+        # see https://stackoverflow.com/questions/63563151/how-to-remove-columns-and-rows-at-certain-indexes-in-numpy-at-the-same-time
+        idx = list(set(range(len(distances))).difference(cluster))
+        distances=distances[np.ix_(idx,idx)]
+        weights=weights[idx]
+        indexes=indexes[idx]
     return ClusteringResult(method="qt",clusters=clusters, weights=ww)
