@@ -16,6 +16,7 @@ import re
 import platform
 import time
 import shlex
+import hashlib
 
 def find_free_port():
     """Returns the number of a free port."""
@@ -24,11 +25,18 @@ def find_free_port():
         s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         return s.getsockname()[1]
 
+_max_sockname=70
+_min_sockname=50
+
 def _adjust_sockname(sockname,port):
     pwd=os.getcwd()
     pwd=re.sub("[^-A-Za-z0-9.]",":",pwd)
     sockname=re.sub(r"\(path\)",pwd,sockname)
     sockname=re.sub(r"\(port\)",str(port),sockname)
+    if len(sockname)>_max_sockname:
+      m=hashlib.blake2b(digest_size=(_max_sockname-_min_sockname-1)//2)
+      m.update(bytes(sockname[_min_sockname:],'utf-8'))
+      sockname=sockname[:_min_sockname] + "-" + m.hexdigest()
     return sockname
 
 def run_server(dry_run: bool = False,
@@ -38,7 +46,8 @@ def run_server(dry_run: bool = False,
                no_screen: bool = False,
                keep_ld_library_path: bool = True,
                python_exec: str = "",
-               sockname: str = "(path):jupyter-server:(port)",
+               sockname: str = "(path):(port):jupyter",
+               lab: bool = False,
                detach: bool = False):
     """Runs a jupyter server inside a screen command.
 
@@ -71,8 +80,11 @@ def run_server(dry_run: bool = False,
             cmd.append("LD_LIBRARY_PATH=" + os.environ["LD_LIBRARY_PATH"])
     cmd.extend(shlex.split(python_exec)) # allows python_exec to contain space separated options
     cmd.append("-m")
-    cmd.append("jupyter")
-    cmd.append("notebook")
+    if lab:
+        cmd.append("jupyterlab")
+    else:
+        cmd.append("jupyter")
+        cmd.append("notebook")
     cmd.append("--no-browser")
     cmd.append("--port=" + str(port))
     print("cmd:", cmd)
@@ -89,7 +101,9 @@ def remote(server: str,
            index: int = 0,
            open_cmd: str = ""):
 
-    cmd = python_exec+" -m jupyter notebook list"
+    cmd =  "( " + python_exec + " -m jupyter notebook list 2> /dev/null ) ;"
+    cmd += "echo x ;"
+    cmd += "( " + python_exec + " -m jupyterlab list 2> /dev/null ) ;"
 
     print("server:", server)
     print("cmd:", cmd)
@@ -101,6 +115,8 @@ def remote(server: str,
         ll = []
         ll_localhost = []
         args = ['ssh', server, cmd]
+        ll_type = []
+        found_lab=False
 
         for l in subprocess.run(args,
                                 stdout=subprocess.PIPE,
@@ -110,9 +126,16 @@ def remote(server: str,
             if re.match("^http", l):
                 ll.append(re.sub("localhost", server, l))
                 ll_localhost.append(l)
+                if found_lab:
+                    ll_type.append("(L)")
+                else:
+                    ll_type.append("(N)")
+
+            if l == "x":
+                found_lab = True
 
         for i in range(len(ll)):
-            print(str(i+1)+") "+ll[i])
+            print(str(i+1)+") "+ll[i]+" "+ll_type[i])
 
         if list_only:
             return
