@@ -7,6 +7,13 @@ import numpy as np
 from scipy.optimize import minimize
 from . import coretools
 
+try:
+    import cudamat as cm  # pylint: disable=import-error
+    _HAS_CUDAMAT=True
+except ModuleNotFoundError:
+    _HAS_CUDAMAT=False
+
+
 class MaxentResult(coretools.Result):
     """Result of a `bussilab.maxent.maxent` calculation."""
     def __init__(self,
@@ -37,6 +44,17 @@ class MaxentResult(coretools.Result):
         self.nit = nit
         """`int` reporting the number of iterations in the minimization procedure."""
 
+# Internal tool
+# should be called before sum() calls in cudamat
+def _ensure_ones(min_size):
+    if cm.CUDAMatrix.ones.shape[0]<min_size:
+        cm.CUDAMatrix.ones = cm.empty((min_size, 1)).assign(1.0)
+
+def _ensure_cm_init():
+    if not hasattr(cm.CUDAMatrix, 'ones'):
+        cm.cublas_init()
+    elif not isinstance(cm.CUDAMatrix.ones,cm.CUDAMatrix):
+        cm.cublas_init()
 
 # Internal tool to compute averages over trajectory.
 # Does not access external data.
@@ -44,20 +62,41 @@ class MaxentResult(coretools.Result):
 def _heavy_part(logW: np.ndarray,
                 traj: np.ndarray,
                 l: np.ndarray,
-                weights: bool = False):
-    logW_ME = logW-np.dot(traj, l)  # maxent correction
-    shift_ME = np.max(logW_ME)  # shift to avoid overflow
-    W_ME = np.exp(logW_ME - shift_ME)
-    # Partition function:
-    Z = np.sum(W_ME)
-    # Averages:
-    averages = np.dot(W_ME, traj) / Z
-    logZ = np.log(Z) + shift_ME
-    if not weights:
-        return (logZ, averages)
-    # only return weights if requested:
-    logW_ME -= np.log(Z)+shift_ME
-    return (logZ, averages, logW_ME)
+                weights: bool = False,
+                cuda: bool = False):
+    if cuda:
+        cu_minus_l=cm.CUDAMatrix(np.reshape(-l,(-1,1)))
+        logW_ME = cm.dot(traj,cu_minus_l)
+        logW_ME.add(logW)
+        shift_ME = logW_ME.max(0).asarray()[0,0]
+        logW_ME.subtract(float(shift_ME))
+        if weights:
+            save_logW_ME=logW_ME.copy()
+        cm.exp(logW_ME)
+        _ensure_ones(logW_ME.shape[0])
+        Z=logW_ME.sum(0).asarray()[0,0]
+        averages = cm.dot(logW_ME.transpose(),traj).asarray()[0,:]
+        averages=np.array(averages,dtype="float64")/Z
+        logZ = np.log(Z) + shift_ME
+        if not weights:
+            return (logZ, averages)
+        save_logW_ME.subtract(float(np.log(Z)))
+        return (logZ, averages, save_logW_ME.asarray()[:,0])
+    else:
+        logW_ME = logW-np.dot(traj, l)  # maxent correction
+        shift_ME = np.max(logW_ME)  # shift to avoid overflow
+        W_ME = np.exp(logW_ME - shift_ME)
+        # Partition function:
+        Z = np.sum(W_ME)
+        # Averages:
+        averages = np.dot(W_ME, traj) / Z
+        logZ = np.log(Z) + shift_ME
+        if not weights:
+            return (logZ, averages)
+        # only return weights if requested:
+        logW_ME -= np.log(Z)+shift_ME
+        return (logZ, averages, logW_ME)
+
 
 def maxent(
         traj,
@@ -72,7 +111,8 @@ def maxent(
         method: str = "L-BFGS-B",
         regularization: Optional[Callable] = None,
         tol: Optional[float] = None,
-        options=None):
+        options=None,
+        cuda=False):
     """Tool that computes new weights to enforce reference values.
 
        This tools process a an array containing the observables computed along a trajectory and
@@ -85,6 +125,8 @@ def maxent(
        traj : array_like
            A 2D array (lists or tuples are internally converted to numpy arrays).
            `traj[i,j]` is j-th observable computed in the i-th frame.
+           If traj is a CUDAMatrix object, then cudamat is used irrespectively of the
+           bool parameter `cuda`.
 
        reference : array_like
 
@@ -127,8 +169,9 @@ def maxent(
 
            A function that takes as argument the current lambdas and return an tuple
            containing the regularization function and its derivatives. For instance,
-           passing a function defined as `def reg(x): return
-           (1e-4*0.5*np.sum(x**2),1e-4*x)` is equivalent to passing `l2=1e-4`.
+           passing a function defined as
+           `def reg(x): return (0.0001*0.5*np.sum(x**2),0.0001*x)`
+           is equivalent to passing `l2=0.0001`.
 
        verbose : bool
            If True, progress informations are written on stdout.
@@ -145,9 +188,39 @@ def maxent(
        options : dict
            Arbitrary options passed to `scipy.optimize.minimize`.
 
+       cuda : bool or None (default False)
+           Use cuda. If None, chosen based on the availability of the cudamat library.
+
+       Notes on using CUDA
+       -------------------
+
+       Note that for normal datasets the cost of transfering the traj object to
+       the GPU dominates. It it however possible to transfer the traj object first to the GPU
+       with `cu_traj=cm.CUDAMatrix(traj)` and then reuse it for multiple calls
+       (e.g. for a hyper parameter scan).
+
     """
 
-    traj = coretools.ensure_np_array(traj)
+    if cuda is None:
+        cuda = _HAS_CUDAMAT
+
+    # when a cudamatrix is passed, cuda is enabled by default
+    if _HAS_CUDAMAT:
+        if isinstance(traj,cm.CUDAMatrix):
+            cuda=True
+
+    if cuda:
+        if not _HAS_CUDAMAT:
+            raise ValueError("Cudamat not available, can only run ANN with numpy")
+        _ensure_cm_init()
+        if isinstance(traj,cm.CUDAMatrix):
+            cu_traj=traj
+        else:
+            traj = coretools.ensure_np_array(traj)
+            cu_traj=cm.CUDAMatrix(traj)
+    else:
+        traj = coretools.ensure_np_array(traj)
+
     lambdas = coretools.ensure_np_array(lambdas)
 
     nframes = traj.shape[0]
@@ -245,6 +318,9 @@ def maxent(
     W0 = np.exp(logW - shift0)
     logZ0 = np.log(np.sum(W0)) + shift0
 
+    if cuda:
+        cu_logW=cm.CUDAMatrix(np.reshape(logW,(-1,1)))
+
     # function to be minimized
     def func(l):
 
@@ -267,11 +343,17 @@ def maxent(
         else:
             ll = l
 
-        logZ, averages = _heavy_part(logW, traj, ll)
+
+        if cuda:
+            logZ, averages = _heavy_part(cu_logW, cu_traj, ll, cuda=cuda)
+        else:
+            logZ, averages = _heavy_part(logW, traj, ll, cuda=cuda)
+
 
         f = logZ - logZ0
         der = -averages
 
+
         if regularization is not None:
             reg = regularization(ll)
             f += reg[0]
@@ -346,10 +428,13 @@ def maxent(
         lambdas = res.x
 
     # recompute weights at the end
-    logZ, averages, logW_ME = _heavy_part(logW, traj, lambdas, weights=True)
+    if cuda:
+        logZ, averages, logW_ME = _heavy_part(cu_logW, cu_traj, lambdas, weights=True, cuda=cuda)
+    else:
+        logZ, averages, logW_ME = _heavy_part(logW, traj, lambdas, weights=True)
 
     if verbose:
-        sys.stderr.write("MAXENT: end")
+        sys.stderr.write("MAXENT: end\n")
 
     return MaxentResult(
         logW_ME=logW_ME,
