@@ -9,17 +9,11 @@ import numpy as np
 from scipy.optimize import minimize
 from typing import Optional, Callable
 import warnings
-
-try:
-    from numba import jit as numba_jit
-except ImportError:
-    warnings.warn("There was a problem importing numba, jit functions will work but will be MUCH slower")
-    def numba_jit(x):
-        return x
-
-
+from .coretools import import_numba_jit
 from . import coretools
 
+numba_jit=import_numba_jit()
+
 @numba_jit
 def _make_lists(size: int,
                 colors: int = 1,
